<?php

use Delo\DebugInfo;
use Delo\XmlReturn;
use Delo\Settings;

class DeloBankApiModuleFrontController extends ModuleFrontController
{
    /** @var DebugInfo */
    private $debugInfo;

    /** @var XmlReturn */
    private $xmlReturn;

    /** @var Settings */
    private $settingsModel;

    public function __construct()
    {
        parent::__construct();
        $this->debugInfo = DebugInfo::getInstance();
        $this->xmlReturn = new XmlReturn();
        $this->settingsModel = new Settings();
    }

    public function postProcess()
    {
        $action = Tools::getValue('action');
        if ('check' === $action) {
            $this->check();
        } elseif ('register' === $action) {
            $this->register();
        } else {
            header('HTTP/1.0 404 Not Found');
            exit();
        }
    }

    public function register()
    {
        $this->debugInfo->set('Request register');
        // Проверить trx_id
        $debugId = $this->debugInfo->set('Проверить наличие trx_id... ');
        if (empty($_GET['trx_id'])) {
            $response = $this->xmlReturn->registerResponseFailed('Пустой параметр trx_id');
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Проверить что только разрешенные символы в trx_id... ');
        $trxId = $_GET['trx_id'];
        $error = $this->settingsModel->pcidValidate($trxId);
        if (!empty($error)) {
            $response = $this->xmlReturn->registerResponseFailed($error);
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Проверить длину, должен быть 32 символа trx_id... ');
        $error = $this->settingsModel->pcidLengthValidate($trxId);
        if (!empty($error)) {
            $response = $this->xmlReturn->registerResponseFailed($error);
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        // Проверить merch_id
        $debugId = $this->debugInfo->set('Проверить наличие merch_id... ');
        $merchId = '';
        if (empty($_GET['merch_id'])) {
            $response = $this->xmlReturn->registerResponseFailed('Пустой параметр merch_id');
            $this->responseXml($response);
        } else {
            $merchId = $_GET['merch_id'];
        }
        $this->debugInfo->setOk($debugId);
        // Проверить id заказа
        $debugId = $this->debugInfo->set('Проверить наличие id заказа в запросе... ');
        if (empty($_GET['o_order_id'])) {
            $response = $this->xmlReturn->registerResponseFailed('Неправильный id заказа');
            $this->responseXml($response);
        }
        $orderId = (int)$_GET['o_order_id'];
        if (empty($orderId)) {
            $response = $this->xmlReturn->registerResponseFailed('id заказа = 0');
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        // Проверить account_id
        $debugId = $this->debugInfo->set('Проверить наличие account_id... ');
        $accountId = '';
        if (empty($_GET['account_id'])) {
            $response = $this->xmlReturn->registerResponseFailed('Empty account id');
            $this->responseXml($response);
        } else {
            $accountId = $_GET['account_id'];
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Сравниваем merch_id с тем что в настройках... ');
        $merchIdFromConfig = Configuration::get(DeloBank::PREFIX . 'merch_id', '');
        if ($merchIdFromConfig != $merchId) {
            $response = $this->xmlReturn->registerResponseFailed('Failed merch_id');
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Сравниваем account_id с тем что в настройках... ');
        $accountIdFromConfig = Configuration::get(DeloBank::PREFIX . 'purchase_account_amount_id', '');
        if ($accountIdFromConfig != $accountId) {
            $response = $this->xmlReturn->registerResponseFailed('Failed account id');
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        // Проверить существование заказа
        $debugId = $this->debugInfo->set('Ищем по id заказ в БД... ');
        try {
            $order = new \Order($orderId);
        } catch (Exception $e) {
            $response = $this->xmlReturn->registerResponseFailed('Внутренняя ошибка, не получается получить заказ. Ошибка:' . $e->getMessage());
            $this->responseXml($response);
        }
        if (empty($order)) {
            $response = $this->xmlReturn->registerResponseFailed('Такого заказа не существует');
            $this->responseXml($response);
        }
        $orderTotalSum = $order->getOrdersTotalPaid();
        $this->debugInfo->setOk($debugId);
        // Проверка существования result_code
        $debugId = $this->debugInfo->set('Проверить наличие result_code... ');
        if (empty($_GET['result_code'])) {
            $response = $this->xmlReturn->registerResponseFailed('Пустой параметр result_code');
            $orderStatusFailedFromConfig = Configuration::get(DeloBank::PREFIX . 'order_status_failed', 0);
            $order->setCurrentState($orderStatusFailedFromConfig);
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        // Проверка amount
        $debugId = $this->debugInfo->set('Проверить наличие amount... ');
        if (empty($_GET['amount'])) {
            $response = $this->xmlReturn->registerResponseFailed('Пустой параметр amount');
            $orderStatusFailedFromConfig = Configuration::get(DeloBank::PREFIX . 'order_status_failed', 0);
            $order->setCurrentState($orderStatusFailedFromConfig);
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Проверить соответствует ли оплаченная сумма той, что в заказе... ');
        $amount = (int)$_GET['amount'];
        $amountOrder = $this->rublesToPennies($orderTotalSum);
        if ($amount != $amountOrder) {
            $response = $this->xmlReturn->registerResponseFailed('Изменилась стоимость заказа на сайте');
            $orderStatusFailedFromConfig = Configuration::get(DeloBank::PREFIX . 'order_status_failed', 0);
            $order->setCurrentState($orderStatusFailedFromConfig);
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        // Цифровая подпись
        $debugId = $this->debugInfo->set('Проверить наличие signature... ');
        if (!empty($_GET['signature'])) {
            $this->debugInfo->setOk($debugId);
            $this->debugInfo->set('Проверка цифровой подписи');
            $checkSignatureFromConfig = Configuration::get(DeloBank::PREFIX . 'check_signature', 0);
            if (!$this->checkSignature($_GET['signature'], $checkSignatureFromConfig)) {
                $response = $this->xmlReturn->registerResponseFailed('Не прошла проверка подлинности ЦП');
                $orderStatusFailedFromConfig = Configuration::get(DeloBank::PREFIX . 'order_status_failed', 0);
                $order->setCurrentState($orderStatusFailedFromConfig);
                $this->responseXml($response);
            }
        } else {
            $response = $this->xmlReturn->registerResponseFailed('Отсутствует ЦП');
            $this->responseXml($response);
        }
        $result = (int)$_GET['result_code'];
        $this->debugInfo->set('result_code = "' . $result . '"');
        if ($result != 1) {
            $this->debugInfo->set('Смотрим расширенный код ошибки и отдаем успешный ответ...');
            if (!empty($_GET['ext_result_code'])) {
                $extResultCode = (string)((int)$_GET['ext_result_code']);
                $error = $this->getErrorByCode($extResultCode);
                if (empty($error)) {
                    $error = 'Неизвестная ошибка платежного шлюза';
                }
            } else {
                $error = 'Неизвестная ошибка платежного шлюза';
            }
            $response = $this->xmlReturn->registerResponseSuccessful($error);
            $orderStatusFailedFromConfig = Configuration::get(DeloBank::PREFIX . 'order_status_failed', 0);
            $order->setCurrentState($orderStatusFailedFromConfig);
            $this->responseXml($response);
        }
        // Установка статуса заказу оплачен
        $debugId = $this->debugInfo->set('Установка статуса заказу оплачен... ');
        $orderStatusAfterFromConfig = Configuration::get(DeloBank::PREFIX . 'order_status_after', 0);
        $order->setCurrentState($orderStatusAfterFromConfig);
        $this->debugInfo->setOk($debugId);
        $this->debugInfo->set('Отдаем успешный ответ... ');
        $response = $this->xmlReturn->registerResponseSuccessful('Заказ № ' . $orderId . ' - успех');
        $this->responseXml($response);
    }

    /**
     * Этот метод ловит первый запрос от платежного шлюза
     */
    private function check()
    {
        $this->debugInfo->set('Request check');
        // Проверить trx_id
        $debugId = $this->debugInfo->set('Проверить наличие trx_id... ');
        if (empty($_GET['trx_id'])) {
            $response = $this->xmlReturn->checkResponseFailed('Пустой параметр trx_id');
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Проверить что только разрешенные символы в trx_id... ');
        $trxId = $_GET['trx_id'];
        $error = $this->settingsModel->pcidValidate($trxId);
        if (!empty($error)) {
            $response = $this->xmlReturn->checkResponseFailed($error);
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Проверить длину, должен быть 32 символа trx_id... ');
        $error = $this->settingsModel->pcidLengthValidate($trxId);
        if (!empty($error)) {
            $response = $this->xmlReturn->checkResponseFailed($error);
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        // Проверить merch_id
        $debugId = $this->debugInfo->set('Проверить наличие merch_id... ');
        if (empty($_GET['merch_id'])) {
            $response = $this->xmlReturn->checkResponseFailed('Пустой параметр merch_id');
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $merchId = $_GET['merch_id'];
        // Проверить id заказа
        $debugId = $this->debugInfo->set('Проверить наличие id заказа в запросе... ');
        if (empty($_GET['o_order_id'])) {
            $response = $this->xmlReturn->checkResponseFailed('Неправильный id заказа');
            $this->responseXml($response);
        }
        $orderId = (int)$_GET['o_order_id'];
        if (empty($orderId)) {
            $response = $this->xmlReturn->checkResponseFailed('id заказа = 0');
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Сравниваем merch_id с тем что в настройках... ');
        $merchIdFromConfig = Configuration::get(DeloBank::PREFIX . 'merch_id', '');
        if ($merchIdFromConfig != $merchId) {
            $response = $this->xmlReturn->checkResponseFailed('Failed merch_id');
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        // Проверить существование заказа и что его стоимость не равна нулю
        $debugId = $this->debugInfo->set('Ищем по id заказ в БД... ');
        try {
            $order = new \Order($orderId);
        } catch (Exception $e) {
            $response = $this->xmlReturn->checkResponseFailed('Внутренняя ошибка, не получается получить заказ. Ошибка:' . $e->getMessage());
            $this->responseXml($response);
        }
        if (empty($order)) {
            $response = $this->xmlReturn->checkResponseFailed('Такого заказа не существует');
            $this->responseXml($response);
        }
        $orderTotalSum = $order->getOrdersTotalPaid();
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Проверить сумму заказа > 0 ... ');
        $amount = $this->rublesToPennies($orderTotalSum);
        if ($amount <= 0) {
            $response = $this->xmlReturn->checkResponseFailed('Стоимость заказа равна нулю');
            $this->responseXml($response);
        }
        $this->debugInfo->setOk($debugId);
        $debugId = $this->debugInfo->set('Все хорошо, готовим положительный ответ... ');
        // @todo: на основе заказа получить информацию о товарах
        // @todo: передавать информацию о покупателе доп. тегах - ФИО, телефон, email
        $shopName = $this->context->shop->name;
        if (empty($shopName)) {
            $shopName = $_SERVER['SERVER_NAME'];
        }
        $response = $this->xmlReturn->checkResponseSuccessful(
            'Проверка платежа заказа № ' . $orderId,
            'Оплата заказа № ' . $orderId,
            'Оплата заказа № ' . $orderId . ' ' . $shopName,
            $amount
        );
        $this->debugInfo->setOk($debugId);
        $this->responseXml($response);
    }

    /**
     * Выставляет заголовок и выплевывает xml
     * @param string $xml
     */
    private function responseXml($xml)
    {
        header('Content-type: application/xml');
        $debugModeEnable = (int)Configuration::get(DeloBank::PREFIX . 'debug_mode', 0);
        if ($debugModeEnable == 1) {
            // Костыль конечно, но пох
            $xml = str_replace('</payment-avail-response>', $this->debugInfo->getXml() . '</payment-avail-response>', $xml);
            $xml = str_replace('</register-payment-response>', $this->debugInfo->getXml() . '</register-payment-response>', $xml);
        }
        exit($xml);
    }

    /**
     * Преобразуем рубли в копейки
     * @param float $sum
     * @return int
     */
    private function rublesToPennies($sum)
    {
        $sum = number_format($sum, 2, '', '');
        return (int)$sum;
    }

    /**
     * Функция для проверки цифровой подписи URI. Возвращает true в случае успеха.
     * @param string $signature ЦП из запроса
     * @param int $enableCheck Вкл./выкл. проверки ЦП
     * @return bool
     * @throws Exception
     */
    public function checkSignature($signature, $enableCheck = 1)
    {
        $debugId = $this->debugInfo->set('Проверка ЦП включена: ');
        if (empty($enableCheck)) {
            $this->debugInfo->setOk($debugId, 'НЕТ');
            // Если проверка ЦП выключена
            return true;
        }
        $this->debugInfo->setOk($debugId, 'ДА');
        $debugId = $this->debugInfo->set('Переменная окружения для определения тестовая это среда или нет ');
        $pathToCert = realpath(__DIR__ . '/../../cert/');
        if (!$pathToCert) {
            $this->debugInfo->set('Не может найти папку с сертификатами, поэтому не проверка не проходит');
            return false;
        }
        if (isset($_SERVER['DELOBANK_ENVIRONMENT'])) {
            $certFile = 'file://' . $pathToCert . '/delobank_test.crt';
            $this->debugInfo->setOk($debugId, 'определена и равна "' . $_SERVER['DELOBANK_ENVIRONMENT'] . '", используем файл "' . $pathToCert . '/delobank_test.crt"');
        } else {
            $certFile = 'file://' . $pathToCert . '/delobank_production.crt';
            $this->debugInfo->setOk($debugId, 'не определена, используем файл "' . $pathToCert . '/delobank_production.crt"');
        }
        $data = $this->getDataFromUrl();
        // Преобразуем подпись в двоичный формат
        $debugId = $this->debugInfo->set('$signature = "' . $signature . '". Преобразуем подпись в двоичный формат... ');
        $signature = base64_decode($signature);
        $this->debugInfo->setOk($debugId);
        // Получаем из сертификата публичный ключ для проверки
        $this->debugInfo->set('Получаем из сертификата публичный ключ для проверки... ');
        $public_key = openssl_pkey_get_public($certFile);
        $this->debugInfo->set(print_r($public_key, true));
        $this->debugInfo->set('ok');
        $debugId = $this->debugInfo->set('Проверяем ЦП... ');
        $check = openssl_verify($data, $signature, $public_key, OPENSSL_ALGO_SHA1);
        $this->debugInfo->setOk($debugId);
        $this->debugInfo->set('Проверка вернула: ' . print_r($check, true));
        openssl_free_key($public_key);
        if ($check !== 1) {
            return false;
        }
        return true;
    }

    /**
     * Выделяет подписанные данные из запроса
     * @return string часть url-а c доменом до &signature=
     */
    private function getDataFromUrl()
    {
        $debugMessage = "Собираем часть url, которая подписана... \n";
        $url = "https://{$_SERVER['HTTP_HOST']}{$_SERVER['REQUEST_URI']}";
        $debugMessage .= 'Полный url: "' . $url . '" ' . "\n";
        $pos = strpos($url, '&signature=');
        $dataUrl = substr($url, 0, $pos);
        $debugMessage .= 'Получили такую часть url-а: "' . $dataUrl . '" ' . "\n";
        $this->debugInfo->set($debugMessage);
        return $dataUrl;
    }

    /**
     * Описание ошибки по ее коду
     * @param string $code код ошибки
     * @return string|null
     */
    private function getErrorByCode($code)
    {
        $arr = [
            '-2' => 'Отказ в проведении платежа',
            '-3' => 'Ошибка при проведении платежа',
            '-4' => 'Успешный платеж отменен по инициативе магазина на этапе фиксации платежа',
            '2'  => 'Магазин отказался проводить платеж на этапе проверки',
            '4'  => 'Произошла ошибка при взаимодействии с магазином на этапе проверки платежа',
            '21' => 'Проверка платежа в магазине не выполнялась. Возможно магазин, или биллинг были заблокированы',
            '53' => 'Транзакция завершилась по таймауту, так как пользователь по каким-то причинам отказался продолжать платеж',
        ];
        if (isset($arr[$code])) {
            return $arr[$code];
        }
        return null;
    }
}
