<?php
namespace Delo;

class Requirements
{
    const PHP_VERSION = '5.4';

    private $errors = [];

    /**
     * @param string $errorText
     */
    public function setError($errorText)
    {
        $this->errors[] = $errorText;
    }

    /**
     * @return array
     */
    public function getErrors()
    {
        return $this->errors;
    }

    /**
     * Проверка требовании для нормальной работы модуля
     * @return bool
     */
    public function check()
    {
        if (!$this->checkVersionPhp() || !$this->checkOpenSsl() || !$this->checkCurl()) {
            return false;
        }
        return true;
    }

    /**
     * Проверка версии PHP
     * @return bool
     */
    private function checkVersionPhp()
    {
        if (!version_compare(PHP_VERSION, self::PHP_VERSION, '>=')) {
            $this->setError('На вашем сервере до сих пор использую PHP ' . PHP_VERSION . ', для модуля "'
                . \DeloBank::MODULE_NAME . '" необходима минимум версия: ' .  self::PHP_VERSION);
            return false;
        }
        return true;
    }

    /**
     * Проверка установлен ли openSSL
     * @return bool
     */
    private function checkOpenSsl()
    {
        if (!extension_loaded('openssl')) {
            $this->setError('Для модуля "' . \DeloBank::MODULE_NAME . '" необходима библиотека openSSL');
            return false;
        }
        return true;
    }

    /**
     * Проверка установлен ли cURL
     * @return bool
     */
    private function checkCurl()
    {
        if (!extension_loaded('curl')) {
            $this->setError('Для модуля "' . \DeloBank::MODULE_NAME . '" необходима библиотека cURL');
            return false;
        }
        return true;
    }
}
